"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zod_1 = require("zod");
const utils_1 = require("../utils");
const error_1 = require("../utils/error");
const http_1 = require("../utils/http");
const schema_1 = require("./schema");
exports.default = ({ strapi }) => ({
    update(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { workflowId } = ctx.params;
                const { approverId, approverType } = ctx.request.body;
                yield Promise.all([
                    (0, utils_1.validateRequest)(Number(workflowId), schema_1.workflowPathParamsSchema),
                    (0, utils_1.validateRequest)(ctx.request.body, schema_1.approverBodySchema)
                ]);
                const entry = yield (0, utils_1.findEntryById)(workflowId);
                if (!entry) {
                    throw new error_1.NotFoundError("No Entry found!");
                }
                const isUserSuperAdmin = (0, utils_1.isSuperAdmin)(ctx.state.user.roles);
                const assignee = entry === null || entry === void 0 ? void 0 : entry.assigneeAsUser;
                const isUserSameAsAssignee = assignee && Number(assignee.id) === Number(approverId);
                if (isUserSameAsAssignee && !isUserSuperAdmin.length) {
                    throw new error_1.ForbiddenError("APPROVER CAN'T be the same as ASSIGNEE.");
                }
                const approver = yield strapi
                    .plugin("review-workflow")
                    .service("approverService")
                    .update({
                    approverId,
                    workflowId,
                    approverType: (0, utils_1.capitalizeFirstLetter)(approverType),
                    entry
                });
                ctx.status = 201;
                ctx.body = new http_1.SuccessResponseObject("Approver updated", approver);
                return ctx;
            }
            catch (err) {
                let status = 500;
                let message = "some error occurred"; // Don't expose db error and other errors internal to code like undefined etc.
                let data;
                if (err instanceof error_1.CustomError) {
                    status = err.statusCode;
                    message = err.message;
                }
                if (err instanceof zod_1.ZodError) {
                    status = 400;
                    message = err.name;
                    data = JSON.parse(err.message);
                }
                yield strapi.plugin("sentry").service("sentry").sendError(JSON.stringify(err));
                ctx.status = status;
                ctx.body = new http_1.ErrorResponseObject(`${message}`, data);
                return ctx;
            }
        });
    }
});
